﻿//'*****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2007                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************/

// -------------------------------------------------------------------
// FVX Sample Program
// メインダイアログ
//
// 作成者 : R.Nagashima
//
// 履歴 : 2007/02/23 初版
// -------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
//FVXデータ構造の使用宣言
using FVDATASLib;


namespace AutoBin
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        // ビデオデバイスがオープンされているか
        private bool m_blnVideoOpened = false;
        // ビデオ設定ファイル名（フルパス）
        private string m_strVideoConfFilePath = "";
        // 初期ディレクトリ
        private string m_strDirectory;
        //処理範囲
        private FvRectangle m_ExeRegion = new FvRectangle();

        private void frmMain_Load(object sender, EventArgs e)
        {
            int IntErr;
            frmSelectvideoConfFile fVideoConf = new frmSelectvideoConfFile();
            frmLicense fLicense = new frmLicense();

            // 変数初期化
            m_blnVideoOpened = false;
            btnGrab.Enabled = false;

            cmbAutoBin.Items.Insert(0, "固定２値化");
            cmbAutoBin.Items.Insert(1, "判別分析法");
            cmbAutoBin.Items.Insert(2, "Pタイル法");
            cmbAutoBin.Items.Insert(3, "ラプラシアンヒストグラム");
            cmbAutoBin.Items.Insert(4, "一次微分ヒストグラム");
            cmbAutoBin.Items.Insert(5, "最小値・最大値範囲内");
            cmbAutoBin.Items.Insert(6, "局所領域平均値");
            cmbAutoBin.Items.Insert(7, "分割領域閾値");
            cmbAutoBin.SelectedIndex = 0;

            //画面の生成
            axFvxDisplayEx1.CreateDisplay();

            //処理範囲の初期値設定（画像の全面に設定）
            m_ExeRegion.xs = -1;
            m_ExeRegion.xe = -1;
            m_ExeRegion.ys = -1;
            m_ExeRegion.ye = -1;

            //作業ディレクトリ退避
            m_strDirectory = Directory.GetCurrentDirectory();

            //使用許諾の確認
            if (fLicense.ShowDialog() == DialogResult.OK)
            {
                //ビデオ設定ファイルの選択
                if (fVideoConf.ShowDialog() == DialogResult.OK)
                    m_strVideoConfFilePath = fVideoConf.strVideoConfFilePath;

                if (fVideoConf.chkNotUseCamera.Checked == false)
                {
                    //ビデオ設定ファイルをコンポーネントに設定
                    axFvxVideo1.VideoConfFile = m_strVideoConfFilePath;

                    //デバイスオープン
                    try
                    {
                        axFvxVideo1.OpenDevice();
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオデバイスのオープンに失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //このため、弊社キャプチャボードFVC02では350msのセットアップタイムが必要になります｡
                    //本プログラムでは簡略化のために、一律に350msのセットアップタイムを確保していますが、
                    //お使いのキャプチャボードに合わせて変更してください。
                    Thread.Sleep(350);

                    //GrabSync/GrabASyncメソッドではImegeサイズの変更を行いませんので、
                    //あらかじめChangeImageSizeメソッドにてImageのサイズを変更してください
                    //（取り込みサイズとImageのサイズが違う場合エラーになります）
                    axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());

                    //２値用に使用するイメージのサイズなどをそろえておく
                    axFvxImage2.ImageType = axFvxImage1.ImageType;
                    axFvxImage2.ImageHeight = axFvxImage1.ImageHeight;
                    axFvxImage2.ImageWidth = axFvxImage1.ImageWidth;
                    
                    try
                    {
                        // 同期取込
                        axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //ビデオデバイスの状態を保存
                    m_blnVideoOpened = true;
                    btnGrab.Enabled = true;

                    //取り込み画像を表示
                    axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
                }
            }
            else
            {
                //終了
                Application.Exit();
                return;
            }
        }

        //終了時の処理
        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (m_blnVideoOpened)
                axFvxVideo1.CloseDevice();
        }
        
        //カメラ取込
        private void btnGrab_Click(object sender, EventArgs e)
        {
            //画像ファイル読み込みによって、画像サイズが初期化時から
            //変更されている可能性があるため、ChangeImageSizeを行います。
            //画像サイズが変更される可能性が無い場合は、初期化時に一度だけ行えば問題有りません。
            axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());

            try
            {
                //同期取込
                axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
            }
            catch (Exception)
            {
                int IntErr;
                axFvxVideo1.GetErrReport(out IntErr);
                MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString());
            }

            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //画像ファイル読込
        private void btnLoadImage_Click(object sender, EventArgs e)
        {
            // ダイアログにて画像ファイルを選択します｡
            openFileDialog1.InitialDirectory = m_strDirectory;
            openFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            openFileDialog1.DefaultExt = "*.bmp";
            openFileDialog1.ShowDialog();

            if (openFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを用いて、
                //ファイルからImageに画像を読み込みます｡
                axFvxFileAccess1.FilePath = openFileDialog1.FileName;

                try
                {
                    axFvxFileAccess1.LoadBitmap(axFvxImage1.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("読込みに失敗しました｡ ErrCode= " + IntErr.ToString());
                    return;
                }

                //画像タイプチェック
                if (axFvxImage1.ImageType != FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray || axFvxImage1.BitPerPixel != 8 )
                {
                    MessageBox.Show("8bitグレイの画像ではありません｡", "error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    axFvxImage1.BitPerPixel = 8;
                    axFvxImage1.ImageType = FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray;
                    axFvxImage1.ImageWidth = 512;
                    axFvxImage1.ImageHeight = 480;
                }
                axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
            }

        }

        //画像ファイル保存
        private void btnSaveImage_Click(object sender, EventArgs e)
        {
            saveFileDialog1.InitialDirectory = m_strDirectory;
            saveFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            saveFileDialog1.DefaultExt = "*.bmp";
            saveFileDialog1.ShowDialog();

            if (saveFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを使って
                //Imageをファイルに保存します｡
                axFvxFileAccess1.FilePath = saveFileDialog1.FileName;
                try
                {
                    axFvxFileAccess1.SaveBitmap(axFvxImage1.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("保存に失敗しました。　ErrCode= " + IntErr.ToString());
                    return;
                }
            }

        }

        //処理範囲設定
        private void btnExeRegion_Click(object sender, EventArgs e)
        {
            int IntOK = 0;
            double dblTemp = 0.0;

            axFvxROI1.SetRectangle(m_ExeRegion);

            // FvxROIを使って処理範囲選択ダイアログを出します｡
            axFvxROI1.UsePoint = false;
            axFvxROI1.MakeROIDialog(axFvxImage1.GetOcx(), out IntOK);

            if ((IntOK == 1) && (axFvxROI1.CheckRectangle == 1))
            {
                axFvxROI1.GetRectangle(m_ExeRegion);
                // 始点と終点が逆になっていないかチェックします｡
                // FvxROIは矩形データで、左上・右下の座標位置(xs > xe, ys > ye)が
                // 逆になっていた場合でも、入れ替えずそのまま結果を返しますので、注意してください
                if (m_ExeRegion.xe < m_ExeRegion.xs)
                {
                    dblTemp = m_ExeRegion.xs;
                    m_ExeRegion.xs = m_ExeRegion.xe;
                    m_ExeRegion.xe = dblTemp;
                }
                if (m_ExeRegion.ye < m_ExeRegion.ys)
                {
                    dblTemp = m_ExeRegion.ys;
                    m_ExeRegion.ys = m_ExeRegion.ye;
                    m_ExeRegion.ye = dblTemp;
                }
            }

        }

        //クリア
        private void btnClear_Click(object sender, EventArgs e)
        {
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

       
        //実行
        private void btnExecute_Click(object sender, EventArgs e)
        {
            double dblTime = 0;
            string ErrMsg;

			int Param1, Param2, Param3, Param4, Param5;
			
            //処理範囲の適用
            axFvxChangeImage1.SetInspectArea(m_ExeRegion);

            //出力側のイメージをクリア
            axFvxImage2.ClearImage();

            try{
                switch(cmbAutoBin.SelectedIndex)
                {
                    case 0://固定２値化
                        if(int.TryParse(txtParam1.Text, out Param1) == false || Param1 < 0 || Param1 > 255)
                        {
                            MessageBox.Show("閾値は 0～255 の間で指定して下さい", "パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
                            return;
                        }
                        axFvxPerformanceCounter1.StartCount();
                        axFvxChangeImage1.FixedLevel(axFvxImage1.GetOcx(),axFvxImage2.GetOcx(), Param1);
                        axFvxPerformanceCounter1.StopCount(out dblTime);
                        break;

                    case 1://判別分析法
                        axFvxPerformanceCounter1.StartCount();
                        axFvxChangeImage1.Discrimination(axFvxImage1.GetOcx(), axFvxImage2.GetOcx());
                        axFvxPerformanceCounter1.StopCount(out dblTime);
                        break;


                    case 2://Ｐタイル法
						if (int.TryParse(txtParam1.Text, out Param1) == false || Param1 < 0 || Param1 > 100)
						{
							MessageBox.Show("２値百分率は 0～100 の間で指定して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
							return;
						}
						axFvxPerformanceCounter1.StartCount();
						axFvxChangeImage1.Ptile(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), Param1);
						axFvxPerformanceCounter1.StopCount(out dblTime);
						break;

					case 3://ラプラシアンヒストグラム
						if (int.TryParse(txtParam1.Text, out Param1) == false || Param1 < 0 || Param1 > 100)
						{
							MessageBox.Show("EdgeFactorは 0～100 の間で指定して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
							return;
						}
						axFvxPerformanceCounter1.StartCount();
						axFvxChangeImage1.LapHist(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(),Param1);
						axFvxPerformanceCounter1.StopCount(out dblTime);
						break;

					case 4://一次微分ヒストグラム
						if (int.TryParse(txtParam1.Text, out Param1) == false || Param1 < 0 || Param1 > 100)
						{
							MessageBox.Show("EdgeFactorは 0～100 の間で指定して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
							return;
						}
						axFvxPerformanceCounter1.StartCount();
						axFvxChangeImage1.GradHist(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(),Param1);
						axFvxPerformanceCounter1.StopCount(out dblTime);
						break;

					case 5://最小値・最大値半内
						if (int.TryParse(txtParam1.Text, out Param1) == false || Param1 < 0 || Param1 > 255)
						{
							MessageBox.Show("最小値は 0～255 の間で指定して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
							return;
						}
						if (int.TryParse(txtParam2.Text, out Param2) == false || Param2 < 0 || Param2 > 255)
						{
							MessageBox.Show("最大値は 0～255 の間で指定して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
							return;
						}
						if(Param1 > Param2)
						{
							MessageBox.Show("最大値≦最小値となるように指定して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
							return;
						}
						axFvxPerformanceCounter1.StartCount();
						axFvxChangeImage1.MinMaxLevel(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(),Param1,Param2);
						axFvxPerformanceCounter1.StopCount(out dblTime);
						break;

					case 6://局所領域平均値
						if (int.TryParse(txtParam1.Text, out Param1) == false || Param1 < 3 || (Param1 % 2) == 0)
						{
							MessageBox.Show("局所領域サイズは 3以上の奇数を指定して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
							return;
						}
						axFvxPerformanceCounter1.StartCount();
						axFvxChangeImage1.ShiftAve(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(),Param1);
						axFvxPerformanceCounter1.StopCount(out dblTime);
						break;

					case 7://分割領域閾値
						if (int.TryParse(txtParam1.Text, out Param1) == false || Param1 < 1 || Param1 > axFvxImage1.ImageWidth - 1)
						{
							ErrMsg = string.Format("横分割数は 1～{0:0}の範囲で入力して下さい",axFvxImage1.ImageWidth - 1);
							MessageBox.Show(ErrMsg,"パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
							return;
						}
						if (int.TryParse(txtParam2.Text, out Param2) == false || Param2 < 1 || Param2 > axFvxImage1.ImageHeight - 1)
						{
							ErrMsg = string.Format("縦分割数は 1～{0:0}の範囲で入力して下さい",axFvxImage1.ImageHeight - 1);
							MessageBox.Show(ErrMsg,"パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
							return;
						}
						if (int.TryParse(txtParam3.Text, out Param3) == false || Param3 < 0 || Param3 > 255)
						{
							MessageBox.Show("MatchAve 0～255 の間で指定して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
							return;
						}
						if (int.TryParse(txtParam4.Text, out Param4) == false || Param4 < 0 || Param4 > 255)
						{
							MessageBox.Show("MatchStddevは 0～255 の間で指定して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
							return;
						}
						if (int.TryParse(txtParam5.Text, out Param5) == false || Param5 <= 0 || Param5 >= 100)
						{
							MessageBox.Show("背景分離度は 1～99 の間で指定して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
							return;
						}
						axFvxPerformanceCounter1.StartCount();
						axFvxChangeImage1.PartDivid( axFvxImage1.GetOcx(),axFvxImage2.GetOcx(),Param1,Param2,Param3,Param4,Param5);
						axFvxPerformanceCounter1.StopCount(out dblTime);
						break;

					default:
						break;
				}

				axFvxDisplayEx1.DrawImage(axFvxImage2.GetOcx());
				txtExecuteTime.Text = dblTime.ToString("0.00ms");
			}
			//COMエラーを取得
			catch(System.Runtime.InteropServices.COMException)
			{
				int ErrNum = 0;
                axFvxChangeImage1.GetErrReport(out ErrNum);

				switch( ErrNum )
				{
					case 0:
						ErrMsg = "エラー無し";
						break;
					case 1501:
						ErrMsg = string.Format("処理に失敗しました。err_no = " + ErrNum.ToString("0") );
						break;
					case 1505:
						ErrMsg = string.Format("パラメータが違います。err_no = " + ErrNum.ToString("0") );
						break;
					case 1506:
						ErrMsg = string.Format("引数の型が違います。err_no = " + ErrNum.ToString("0") );
						break;
					case 1507:
						ErrMsg = string.Format("引数にNULLが指定されています。err_no = " + ErrNum.ToString("0") );
						break;
					case 1531:
						ErrMsg = string.Format("検査対象イメージのＸサイズは、32の倍数でなければなりません。err_no = " + ErrNum.ToString("0") );
						break;
					case 1532:
						ErrMsg = string.Format("検査対象イメージと結果格納イメージは、同一サイズでなければなりません。err_no = "+ ErrNum.ToString("0") );
						break;
					case 1550:
						ErrMsg = string.Format("イメージの種類が違います。err_no = " + ErrNum.ToString("0") );
						break;
					case 1551:
						ErrMsg = string.Format("検査対象イメージと結果格納イメージに、同一のオブジェクトが指定されています。err_no = " + ErrNum.ToString("0") );
						break;
					case 1552:
						ErrMsg = string.Format("処理範囲の指定に間違いがあります。err_no = " + ErrNum.ToString("0") );
						break;
					default:
						ErrMsg = string.Format("不明なエラー err_no = " + ErrNum.ToString("0") );
						break;
				}
				MessageBox.Show(ErrMsg, "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}

			//システムエラーを取得
			catch( Exception ex)
			{
				MessageBox.Show(ex.Message,"システムエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
			}
        }
        
        private void cmbAutoBin_SelectedIndexChanged(object sender, EventArgs e)
        {
            labelParm1.Text = "Param1";
            labelParm2.Text = "Param2";
            labelParm3.Text = "Param3";
            labelParm4.Text = "Param4";
            labelParm5.Text = "Param5";
            labelParm1.Visible = false;
            labelParm2.Visible = false;
            labelParm3.Visible = false;
            labelParm4.Visible = false;
            labelParm5.Visible = false;

            txtParam1.Text = "0";
            txtParam2.Text = "0";
            txtParam3.Text = "0";
            txtParam4.Text = "0";
            txtParam5.Text = "0";
            txtParam1.Visible = false;
            txtParam2.Visible = false;
            txtParam3.Visible = false;
            txtParam4.Visible = false;
            txtParam5.Visible = false;

            switch (cmbAutoBin.SelectedIndex)
            {
                case 0://固定２値化
                    labelParm1.Text = "閾値";
                    txtParam1.Text = "128";
                    labelParm1.Visible = true;
                    txtParam1.Visible = true;
                    break;

                case 1://判別分析法
                    break;

                case 2://Ｐタイル法
                    labelParm1.Text = "２値化百分率";
                    txtParam1.Text = "50";
                    labelParm1.Visible = true;
                    txtParam1.Visible = true;
                    break;

                case 3://ラプラシアンヒストグラム
                    labelParm1.Text = "EdgeFactor";
                    txtParam1.Text = "50";
                    labelParm1.Visible = true;
                    txtParam1.Visible = true;
                    break;

                case 4://一次微分ヒストグラム
                    labelParm1.Text = "EdgeFactor";
                    txtParam1.Text = "50";
                    labelParm1.Visible = true;
                    txtParam1.Visible = true;
                    break;

                case 5://最小値・最大値範囲内
                    labelParm1.Text = "最小値";
                    txtParam1.Text = "96";
                    labelParm2.Text = "最大値";
                    txtParam2.Text = "192";
                    labelParm1.Visible = true;
                    labelParm2.Visible = true;
                    txtParam1.Visible = true;
                    txtParam2.Visible = true;
                    break;

                case 6://局所領域平均値
                    labelParm1.Text = "局所領域サイズ";
                    txtParam1.Text = "25";
                    labelParm1.Visible = true;
                    txtParam1.Visible = true;
                    break;

                case 7://分割領域閾値
                    labelParm1.Text = "横分割数";
                    txtParam1.Text = "10";
                    labelParm2.Text = "縦分割数";
                    txtParam2.Text = "10";
                    labelParm3.Text = "MatchAve";
                    txtParam3.Text = "10";
                    labelParm4.Text = "MatchStddev";
                    txtParam4.Text = "10";
                    labelParm5.Text = "背景分離度";
                    txtParam5.Text = "30";

                    labelParm1.Visible = true;
                    labelParm2.Visible = true;
                    labelParm3.Visible = true;
                    labelParm4.Visible = true;
                    labelParm5.Visible = true;
                    txtParam1.Visible = true;
                    txtParam2.Visible = true;
                    txtParam3.Visible = true;
                    txtParam4.Visible = true;
                    txtParam5.Visible = true;
                    break;

                default:
                    break;
           }
        } 
		
        //エディットボックスに数字と削除ボタン以外入力不可にする
        private void txtParam1_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;

        }

        private void txtParam2_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;

        }

        private void txtParam3_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;

        }

        private void txtParam4_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;

        }

        private void txtParam5_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;

        }     
    }
}